use nom::branch::alt;
use nom::bytes::complete::tag;
use nom::character::complete::{alphanumeric1, char, multispace0, one_of};
use nom::combinator::{not, value};
use nom::multi::{many0, many1};
use nom::sequence::{delimited, terminated};
use nom::IResult;
use nom_locate::LocatedSpan;

pub type Span<'a> = LocatedSpan<&'a str>;
pub type OwnedSpan<'a> = LocatedSpan<String>;

pub fn raw_true(input: Span) -> IResult<Span, bool> {
	value(true, tag("true"))(input)
}

pub fn raw_false(input: Span) -> IResult<Span, bool> {
	value(false, tag("false"))(input)
}

pub fn raw_decimal(input: Span) -> IResult<Span, OwnedSpan> {
	let input_offset = input.location_offset();
	let input_line = input.location_line();

	let (input, list) = many1(terminated(one_of("0123456789"), many0(char('_'))))(input)?;
	let string = list.iter().fold(String::with_capacity(list.len()), |a, b| {
		format!("{}{}", a, b)
	});

	unsafe {
		Ok((
			input,
			OwnedSpan::new_from_raw_offset(input_offset, input_line, string, ()),
		))
	}
}
pub fn tag_ws<'a, 'b: 'a>(tag_val: &'b str, sp: Span<'a>) -> IResult<Span<'a>, Span<'a>> {
	delimited(
		multispace0,
		tag(tag_val),
		not(alt((alphanumeric1, tag("_")))),
	)(sp)
}
