use crate::lexer::atoms::tag_ws;
use crate::lexer::{ScriptToken, ScriptTokenType, Span};
use crate::parse_ops;
use nom::IResult;
use nom_locate::position;

parse_ops!(
	tag_ws ;
	(token_struct, "struct" => ScriptTokenType::Class),
	(token_else, "else" => ScriptTokenType::Else),
	(token_function, "fn" => ScriptTokenType::Function),
	(token_for, "for" => ScriptTokenType::For),
	(token_if, "if" => ScriptTokenType::If),
	(token_null, "null" => ScriptTokenType::Null),
	(token_print, "print" => ScriptTokenType::Print),
	(token_return, "return" => ScriptTokenType::Return),
	(token_super, "super" => ScriptTokenType::Super),
	(token_this, "this" => ScriptTokenType::This),
	(token_let, "let" => ScriptTokenType::Let),
	(token_while, "while" => ScriptTokenType::While),
	(token_export, "export" => ScriptTokenType::Export),
	(token_import, "import" => ScriptTokenType::Import),
	(token_alias, "as" => ScriptTokenType::Alias),
	(token_from, "from" => ScriptTokenType::From),
	(token_typeof, "typeof" => ScriptTokenType::Typeof),
	(token_finally, "finally" => ScriptTokenType::Finally)
);

#[cfg(test)]
mod keyword_checks {
	use super::*;
	use crate::lexer::Span;
	use test_case::test_case;

	fn s(st: &str) -> Span {
		Span::new(st)
	}

	#[test_case(token_struct, "struct" => Ok(ScriptTokenType::Class) ; "Scan token_struct")]
	#[test_case(token_else, "else" => Ok(ScriptTokenType::Else) ; "Scan token_else")]
	#[test_case(token_function, "fn" => Ok(ScriptTokenType::Function) ; "Scan token_function")]
	#[test_case(token_for, "for" => Ok(ScriptTokenType::For) ; "Scan token_for")]
	#[test_case(token_if, "if" => Ok(ScriptTokenType::If) ; "Scan token_if")]
	#[test_case(token_null, "null" => Ok(ScriptTokenType::Null) ; "Scan token_null")]
	#[test_case(token_print, "print" => Ok(ScriptTokenType::Print) ; "Scan token_print")]
	#[test_case(token_return, "return" => Ok(ScriptTokenType::Return) ; "Scan token_return")]
	#[test_case(token_super, "super" => Ok(ScriptTokenType::Super) ; "Scan token_super")]
	#[test_case(token_this, "this" => Ok(ScriptTokenType::This) ; "Scan token_this")]
	#[test_case(token_let, "let" => Ok(ScriptTokenType::Let) ; "Scan token_let")]
	#[test_case(token_while, "while" => Ok(ScriptTokenType::While) ; "Scan token_while")]
	#[test_case(token_export, "export" => Ok(ScriptTokenType::Export) ; "Scan token_export")]
	#[test_case(token_import, "import" => Ok(ScriptTokenType::Import) ; "Scan token_import")]
	#[test_case(token_alias, "as" => Ok(ScriptTokenType::Alias) ; "Scan token_alias")]
	#[test_case(token_from, "from" => Ok(ScriptTokenType::From) ; "Scan token_from")]
	#[test_case(token_typeof, "typeof" => Ok(ScriptTokenType::Typeof) ; "Scan token_typeof")]
	#[test_case(token_finally, "finally" => Ok(ScriptTokenType::Finally) ; "Scan token_finally")]
	fn parse_token(
		func: impl Fn(Span) -> IResult<Span, ScriptToken>,
		tok: &str,
	) -> Result<ScriptTokenType, nom::Err<nom::error::Error<Span>>> {
		func(s(tok)).map(|t| t.1.token_type)
	}
}
