use crate::lexer::{ScriptToken, ScriptTokenType};
use peg::{Parse, ParseElem, ParseLiteral, ParseSlice, RuleResult};
use std::ops::Deref;

#[repr(transparent)]
pub struct TokenSlice<'a>(pub &'a [ScriptToken<'a>]);
impl<'a> Deref for TokenSlice<'a> {
	type Target = [ScriptToken<'a>];
	fn deref(&self) -> &'a Self::Target {
		self.0
	}
}

impl<'a> Parse for TokenSlice<'a> {
	type PositionRepr = usize;

	fn start(&self) -> usize {
		0
	}

	fn is_eof(&self, position: usize) -> bool {
		position >= self.len()
	}

	fn position_repr(&self, position: usize) -> Self::PositionRepr {
		position
	}
}
impl<'a> ParseElem<'a> for TokenSlice<'a> {
	type Element = &'a ScriptToken<'a>;

	fn parse_elem(&'a self, pos: usize) -> RuleResult<Self::Element> {
		match self[pos..].first() {
			Some(elemt) => RuleResult::Matched(pos + 1, elemt),
			None => RuleResult::Failed,
		}
	}
}
impl<'a> ParseLiteral for TokenSlice<'a> {
	fn parse_string_literal(&self, pos: usize, literal: &str) -> RuleResult<()> {
		let matches = self
			.get(pos)
			.map(|token| ScriptTokenType::try_from(literal).as_ref() == Ok(&token.token_type))
			.unwrap_or(false);
		if matches {
			RuleResult::Matched(pos + 1, ())
		} else {
			RuleResult::Failed
		}
	}
}
impl<'a> ParseSlice<'a> for TokenSlice<'a> {
	type Slice = &'a [ScriptToken<'a>];

	fn parse_slice(&'a self, p1: usize, p2: usize) -> Self::Slice {
		&self[p1..p2]
	}
}
