mod printer;
mod simple;

use crate::parser::ast::{Expression, Program, ValueExpression, VoidExpression};
use std::error::Error;

pub trait Visitor {
	type Output;
	type Error: Error;

	fn evaluate_value_expression(
		&mut self,
		expression: &ValueExpression,
	) -> Result<Self::Output, Self::Error>;
	fn evaluate_void_expression(
		&mut self,
		expression: &VoidExpression,
	) -> Result<Self::Output, Self::Error>;
	fn evaluate_expression(
		&mut self,
		expression: &Expression,
	) -> Result<Self::Output, Self::Error> {
		match expression {
			Expression::Value(expr) => self.evaluate_value_expression(expr),
			Expression::Void(expr) => self.evaluate_void_expression(expr),
		}
	}
	fn evaluate_program(&mut self, program: &Program) -> Result<Self::Output, Self::Error>;
}

pub use printer::TreePrinter;
