use crate::runtime::value::ForgeValue;
use crate::utilities::Clown;
use std::ops::{Deref, DerefMut};

#[derive(Default)]
pub struct ConstData(Vec<ForgeValue>);
pub struct ConstDataRef<'scope>(&'scope [ForgeValue]);

impl<'scope> Clown<ConstData> for ConstDataRef<'scope> {
	fn clown(&self) -> ConstData {
		ConstData(self.0.to_vec())
	}
}

impl From<Vec<ForgeValue>> for ConstData {
	fn from(value: Vec<ForgeValue>) -> Self {
		ConstData(value)
	}
}

impl<'scope> From<&'scope [ForgeValue]> for ConstDataRef<'scope> {
	fn from(value: &'scope [ForgeValue]) -> Self {
		ConstDataRef(value)
	}
}

impl Deref for ConstData {
	type Target = Vec<ForgeValue>;

	fn deref(&self) -> &Self::Target {
		&self.0
	}
}
impl DerefMut for ConstData {
	fn deref_mut(&mut self) -> &mut Self::Target {
		&mut self.0
	}
}
impl ConstData {
	pub fn as_ref(&self) -> ConstDataRef {
		ConstDataRef(self.0.as_slice())
	}
}

impl<'scope> Deref for ConstDataRef<'scope> {
	type Target = &'scope [ForgeValue];

	fn deref(&self) -> &Self::Target {
		&self.0
	}
}
impl<'scope> DerefMut for ConstDataRef<'scope> {
	fn deref_mut(&mut self) -> &mut Self::Target {
		&mut self.0
	}
}
