use forge_script_lang::runtime::vm::{Chunk, Forge};
use std::io::Write;
use std::num::ParseIntError;

pub struct Repl {
	buffer: Vec<String>,
	vm: Forge,
}

fn parse_hex(string: &str) -> Result<Vec<u8>, ParseIntError> {
	let parts = string.split(' ').collect::<Vec<&str>>();
	let mut out = Vec::with_capacity(parts.len());
	for part in parts {
		out.push(u8::from_str_radix(part, 16)?);
	}
	Ok(out)
}

impl Repl {
	pub fn new() -> Self {
		Repl {
			buffer: Vec::new(),
			vm: Forge::from_chunk(Chunk::default()),
		}
	}

	pub fn run(&mut self) {
		println!("\n] =========== [");
		println!("] Forgescript [");
		println!("] =========== [");
		println!("\nRepl v{}\n", env!("CARGO_PKG_VERSION"));

		'repl_loop: loop {
			print!("> ");
			match std::io::stdout().flush() {
				Ok(_) => {}
				Err(e) => {
					eprintln!("{}", e);
					break 'repl_loop;
				}
			}

			let mut buffer = String::new();
			let stdin = std::io::stdin();

			match stdin.read_line(&mut buffer) {
				Ok(_) => {}
				Err(e) => {
					eprintln!("\n{}", e);
					break 'repl_loop;
				}
			}

			let buffer = buffer.trim();
			match buffer {
				".quit" => {
					break 'repl_loop;
				}
				".history" => {
					println!("\n== History ==");
					for line in self.buffer.iter() {
						println!("{}", line);
					}
					println!("=============\n");
				}
				other => {
					self.buffer.push(String::from(other));
					println!(
						"{}",
						match Forge::compile_and_run(other) {
							Ok(val) => format!("{}", val),
							Err(e) => format!("{}", e),
						}
					);
				}
			}
		}
	}
}
